from typing import Any, Dict, Type
import time

ASTERISK_BANNER: str = 'Asterisk Call Manager'

class Operation:
    _asterisk_name: str = ''
    _label: str = ''

    def __init__(self, **kwargs):
        self._name = None
        self._list_id: int = None
        self._raw: str = ''
        self._dict: Dict[str, Any] = {}

        self._timestamp: float = time.time()

        self._dict.update(kwargs)
        self._raw = self.convert_to_raw_content(self._dict)
        self._init_name()

    def _init_name(self) -> None:
        if hasattr(self, 'action'):
            self._name = self.action
        
        elif hasattr(self, 'event'):
            self._name = self.event
        
        elif hasattr(self, 'response'):
            self._name = self.response
        
        else:
            raise 'Operation must have one of this attributes: action, event, response'

    @staticmethod
    def parse_raw_content(raw: str) -> Dict[str, Any]:
        lines = raw.strip().split('\r\n')
        operation_dict: Dict[str, Any] = {}
        for line in lines:
            if ASTERISK_BANNER in line: continue

            key, value = line.split(':', 1)
            operation_dict[key.lstrip()] = value.lstrip()

        return operation_dict

    @staticmethod
    def convert_to_raw_content(operation_dict: Dict[str, Any]) -> str:
        raw_operation: str = ''
        for key, value in operation_dict.items():
            raw_operation += f'{key.replace('_', '-')}: {value}\r\n'

        raw_operation += '\r\n'
        return raw_operation

    def __str__(self) -> str:
        return f'<Operation: {self._asterisk_name}>'

    def __repr__(self) -> str:
        return f'<Operation: {self._asterisk_name}>'
